#!/usr/bin/perl -w

use strict;
use FDN::Common::FileLock;
use FDN::Common::Constantes;
use IO::Handle;
use English;

$OUTPUT_AUTOFLUSH=1;
my $filename = "/tmp/fdn_common_test.lock";
my $fh = new IO::Handle;
open $fh, ">", $filename;

my @childs;

print "1..10\n";

my $cld1 = init_child1();
my $cld2 = init_child2();
my $cld3 = init_child3();

# On attend le temps que les fils ouvrent leurs filehandles
sleep 2;

child_lock($cld1);
child_lock($cld2);
child_lock($cld3);
child_unlock($cld1);
child_lock($cld3);
child_lock($cld2);
child_unlock($cld2);
child_lock($cld1);
child_unlock($cld1);
child_unlock($cld3);

foreach my $pid ($cld1, $cld2, $cld3) {
#	print "Kill du fils $pid\n";
	close_child($pid)
}

sub pose_lock($$$$) {
	my ( $fh, $cld, $n, $exp ) = @_;
	my $res = setlock($fh);
	if ( $res == $exp ) {
		print "ok $n - $cld setlock $exp\n";
	} else {
		print "not ok $n - $cld setlock expected:$exp got:$res\n";
	}
}

sub retire_lock($$$$) {
	my ( $fh, $cld, $n, $exp ) = @_;
	my $res = unsetlock($fh);
	if ( $res == $exp ) {
		print "ok $n - $cld unsetlock $exp\n";
	} else {
		print "not ok $n - $cld unsetlock expected:$exp got:$res\n";
	}
}

sub init_child1 {
	if (my $pid = fork) {
		return $pid;
	} else {
		sub c1_usr1_1() {
			# 1 / prise du lock par child1.
			pose_lock($fh,'cld1',1,TRUE);
			$SIG{'USR1'} = \&c1_usr1_2;
		}
		sub c1_usr2_1() {
			# 4 / retrait du lock par child1
			retire_lock($fh,'cld1',4,1);
			$SIG{'USR2'} = \&c1_usr2_2;
		}
		sub c1_usr1_2() {
			# 8 / refus du lock pour cld1 (tenu par cld3)
			pose_lock($fh,'cld1',8,FALSE);
			$SIG{'USR1'} = \&c1_usr1_3;
		}
		sub c1_usr2_2() {
			# 9 / retrait du lock qu'on n'a pas (ca marche, connement)
			retire_lock($fh,'cld1',9,TRUE);
			$SIG{'USR2'} = \&c1_usr2_3;
		}
		sub c1_usr1_3() {
		}
		sub c1_usr2_3() {
		}

		local $SIG{'USR1'} = \&c1_usr1_1;
		local $SIG{'USR2'} = \&c1_usr2_1;

		while (1) {
			#print "cld1 Waiting\n";
			sleep();
		}
		die;
	}
}

sub init_child2 {
	if (my $pid = fork) {
		return $pid;
	} else {
		#my $fh = new IO::Handle;
		#open $fh, ">", $filename;
		sub c2_usr1_1() {
			# 2 / refus du lock pour cld2 (tenu par cld1)
			pose_lock($fh,'cld2',2,FALSE);
			$SIG{'USR1'} = \&c2_usr1_2;
		}
		sub c2_usr1_2() {
			# 6 / refus du lock pour cld2 (tenu par cld3)
			pose_lock($fh,'cld2',6,FALSE);
			$SIG{'USR1'} = \&c2_usr1_3;
		}
		sub c2_usr2_1() {
			# 7 / retrait du lock qu'on ne detient pas.
			retire_lock($fh,'cld2',7,TRUE);
			$SIG{'USR2'} = \&c2_usr2_2;
		}
		sub c2_use1_3() {
		}
		sub c2_usr2_2() {
		}
		
		local $SIG{'USR1'} = \&c2_usr1_1;
		local $SIG{'USR2'} = \&c2_usr2_1;

		while (1) {
			#print "cld2 Waiting\n";
			sleep();
		}
		die;
	}
}

sub init_child3 {
	if (my $pid = fork) {
		return $pid;
	} else {
		#my $fh = new IO::Handle;
		#open $fh, ">", $filename;
		sub c3_usr1_1() {
			# 3 / refus du lock pour cld3
			pose_lock($fh,'cld3',3,FALSE);
			$SIG{'USR1'} = \&c3_usr1_2;
		}
		sub c3_usr1_2() {
			# 5 / pris du lock par cld3
			pose_lock($fh,'cld3',5,TRUE);
			$SIG{'USR1'} = \&c3_usr1_3;
		}
		sub c3_usr2_1() {
			# 10 / retrait du lock pris au test 5.
			retire_lock($fh,'cld3',10,TRUE);
			$SIG{'USR1'} = \&c3_usr2_2;
		}
		sub c3_usr1_3() {
		}
		sub c3_usr2_2() {
		}

		local $SIG{'USR1'} = \&c3_usr1_1;
		local $SIG{'USR2'} = \&c3_usr2_1;

		while (1) {
			#print "cld3 Waiting\n";
			sleep();
		}
		die;
	}
}

sub close_child {
	my ($pid) = @_;

	kill 3, $pid;
	waitpid $pid, 0;
}

sub child_lock {
	my ($pid) = @_;
	#print "demande de lock  $pid\n";
	kill 10, $pid;
	sleep 1;
}

sub child_unlock {
	my ($pid) = @_;
	#print "demande de unlock  $pid\n";
	kill 12, $pid;
	sleep 1;
}
